# frozen_string_literal: true

module ActionPush
  class Base
    extend Concerns::ClassMethodDelivery
    extend Concerns::RegisterDefault
    extend Concerns::RegisterDeliveryMethod
    extend Concerns::RegisterInterceptor

    register_default :ios, Ios::Message.new

    attr_accessor :action_name

    def self.inherited(subclass)
      super

      subclass.register_interceptor(interceptor)

      defaults.each do |name, placeholder|
        subclass.register_default(name, placeholder.clone)
      end

      delivery_methods.each do |name, handler|
        subclass.register_delivery_method(name, handler)
      end
    end

    def initialize(action_name: nil)
      @action_name = action_name
    end

    def envelope
      @envelope ||= Envelope.new do |env|
        self.class.defaults.each do |name, placeholder|
          env.add(name, placeholder.dup)
        end
      end
    end

    # @return [Provider::Envelope]
    def push(provider, attributes = {})
      push = envelope.for(provider)

      push.title = envelope.title if envelope.title
      push.body = envelope.body if envelope.body
      push.payload = envelope.payload if envelope.payload
      push.scheduled = true
      push.delivery_method = self.class.delivery_methods.fetch(provider, Delivery::Explosion)

      # after all
      push.assign(attributes)
      yield(push) if block_given?

      envelope
    end

    def translate(key, options = {})
      I18n.translate(key, { scope: i18n_scope }.merge(options))
    end

    def i18n_scope
      "#{Util.underscore(self.class.to_s)}.#{action_name}"
    end

    alias_method :t, :translate
  end
end
