# frozen_string_literal: true

module ActionPush
  module Concerns
    module ClassMethodDelivery
      # Acts like ActionMailer::Base
      #
      # class ApplePush < ActionPush
      #   def welcome
      #     # ....
      #   end
      # end
      #
      # ApplePush.new.welcome will build PUSH message, but not send it
      # ApplePush.welcome will build PUSH message and DELIVER it
      #
      # @return [Envelope]
      def method_missing(method, *argv, &block)
        return super unless action_methods.include?(method)

        instance = new(action_name: method)
        instance.public_send(method, *argv, &block)
        instance.envelope.pushes.each do |provider, push|
          interceptor.call(instance, provider, push) do
            push.deliver
          end
        end

        instance.envelope
      end

      def respond_to_missing?(method_name, include_private = false)
        action_methods.include?(method_name) || super
      end

      def action_methods
        @action_methods ||= public_instance_methods(false).to_set
      end
    end
  end
end
