# frozen_string_literal: true

module ActionPush
  module Ios
    # @refs https://developer.apple.com/library/archive/documentation/NetworkingInternet/Conceptual/RemoteNotificationsPG/PayloadKeyReference.html#//apple_ref/doc/uid/TP40008194-CH17-SW5
    class Alert
      # [String]
      # A short string describing the purpose of the notification.
      # Apple Watch displays this string as part of the notification interface.
      # This string is displayed only briefly and should be crafted so that
      # it can be understood quickly.
      attr_accessor :title

      # [String]
      # The text of the alert message
      attr_accessor :body

      # [String, nil]
      # The key to a title string in the Localizable.strings file for the current localization.
      # The key string can be formatted with %@ and %n$@ specifiers to take the variables specified in
      # the title-loc-args array
      attr_accessor :title_loc_key

      # [Array<String>, nil]
      # Variable string values to appear in place of the format specifiers in title-loc-key
      attr_accessor :title_loc_args

      # [String, nil]
      # If a string is specified, the system displays an alert that includes the Close and View buttons.
      # The string is used as a key to get a localized string in the current localization to use for the
      # right button’s title instead of “View”
      attr_accessor :action_loc_key

      # [String]
      # A key to an alert-message string in a Localizable.strings file for the current localization
      # (which is set by the user’s language preference). The key string can be formatted with
      # %@ and %n$@ specifiers to take the variables specified in the loc-args array
      attr_accessor :loc_key

      # [Array<String>]
      # Variable string values to appear in place of the format specifiers in loc-key.
      attr_accessor :loc_args

      # [String]
      # The filename of an image file in the app bundle, with or without the filename extension.
      # The image is used as the launch image when users tap the action button or move the action slider
      attr_accessor :launch_image
    end
  end
end
