# frozen_string_literal: true

module ActionPush
  module Provider
    class Base
      # [Boolean]
      # If the PUSH message scheduled to deliver
      attr_accessor :scheduled

      # [String]
      # Device token
      attr_accessor :token

      # extra data (JSON) to sent with a PUSH
      # message as a payload
      attr_accessor :payload

      # [Proc]
      # object which is responds to #call
      attr_accessor :delivery_method

      # [any]
      # response from the delivery gate
      attr_accessor :delivery_response

      # [Boolean]
      # +true+ if push was successfully sent to the delivery gate
      attr_accessor :delivered

      def initialize(params = {})
        assign(params)

        yield(self) if block_given?
      end

      # @return [Self]
      def assign(params = {})
        params.each { |key, value| public_send("#{key}=", value) }

        self
      end

      def title=(_value)
        raise NotImplementedError, __method__
      end

      def body=(_value)
        raise NotImplementedError, __method__
      end

      def title
        raise NotImplementedError, __method__
      end

      def body
        raise NotImplementedError, __method__
      end

      def deliver
        delivery_method.call(self) if scheduled
      end
    end
  end
end
